@@ -3,6 +3,7 @@ Infer ancestral sequences based on a tree.
 """
 
 import os, shutil, time, json, sys
+import numpy as np
 from Bio import Phylo, SeqIO
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
@@ -60,7 +61,7 @@ def ancestral_sequence_inference(tree=None, aln=None, ref=None, infer_gtr=True,
 
     return tt
 
-def collect_mutations_and_sequences(tt, infer_tips=False, full_sequences=False, character_map=None):
+def collect_mutations_and_sequences(tt, infer_tips=False, full_sequences=False, character_map=None, mask_ambiguous=True):
     """iterates of the tree and produces dictionaries with
     mutations and sequences for each node.
 
@@ -91,14 +92,29 @@ def collect_mutations_and_sequences(tt, infer_tips=False, full_sequences=False,
         data[n.name]['muts'] = [a+str(int(pos)+inc)+cm(d)
                                 for a,pos,d in n.mutations]
 
+    mask=None
     if full_sequences:
+        if mask_ambiguous:
+            # Identify sites for which every terminal sequence is ambiguous.
+            # These sites will be masked to prevent rounding errors in the
+            # maximum likelihood inference from assigning an arbitrary
+            # nucleotide to sites at internal nodes.
+            ambiguous_count = np.zeros(tt.sequence_length, dtype=int)
+            for n in tt.tree.get_terminals():
+                ambiguous_count += np.array(tt.sequence(n,reconstructed=False, as_string=False)==tt.gtr.ambiguous, dtype=int)
+            mask = ambiguous_count==len(tt.tree.get_terminals())
+        else:
+            mask = np.zeros(tt.sequence_length, dtype=bool)
+
         for n in tt.tree.find_clades():
             try:
-                data[n.name]['sequence'] = tt.sequence(n,reconstructed=infer_tips, as_string=True)
+                tmp = tt.sequence(n,reconstructed=infer_tips, as_string=False)
+                tmp[mask] = tt.gtr.ambiguous
+                data[n.name]['sequence'] = "".join(tmp)
             except:
                 print("No sequence available for node ",n.name)
 
-    return data
+    return {"nodes": data, "mask": mask}
 
 
 def register_arguments(parser):
@@ -152,7 +168,7 @@ def run(args):
     else:
         aln = args.alignment
 
-    # Enfore treetime 0.7 or later
+    # Enforce treetime 0.7 or later
     from distutils.version import StrictVersion
     import treetime
     if StrictVersion(treetime.version) < StrictVersion('0.7.0'):
@@ -177,14 +193,20 @@ def run(args):
         else:
             character_map[x] = x
 
-    anc_seqs['nodes'] = collect_mutations_and_sequences(tt, full_sequences=not is_vcf,
-                            infer_tips=infer_ambiguous, character_map=character_map)
+    anc_seqs.update(collect_mutations_and_sequences(tt, full_sequences=not is_vcf,
+                          infer_tips=infer_ambiguous, character_map=character_map))
     # add reference sequence to json structure. This is the sequence with
     # respect to which mutations on the tree are defined.
     if is_vcf:
         anc_seqs['reference'] = {"nuc":compress_seq['reference']}
     else:
-        anc_seqs['reference'] = {"nuc":"".join(T.root.sequence) if hasattr(T.root, 'sequence') else ''}
+        root_seq = tt.sequence(T.root, as_string=False)
+        if anc_seqs.get("mask") is not None:
+            root_seq[anc_seqs['mask']] = tt.gtr.ambiguous
+        anc_seqs['reference'] = {"nuc": ''.join(root_seq)}
+
+    if anc_seqs.get("mask") is not None:
+        anc_seqs["mask"] = "".join(['1' if x else '0' for x in anc_seqs["mask"]])
 
     out_name = get_json_name(args, '.'.join(args.alignment.split('.')[:-1]) + '_mutations.json')
     write_json(anc_seqs, out_name)
