@@ -35,14 +35,11 @@ calculations, limiting the computationally complexity of the comparisons.
 Distance maps
 =============
 
-Distance maps are defined in JSON format with two required top-level keys. The
-`default` key specifies the numeric value (integer or float) to assign to all
-mismatches by default. The `map` key specifies a dictionary of weights to use
-for distance calculations. These weights are indexed hierarchically by gene name
-and one-based gene coordinate and are assigned in either a sequence-independent
-or sequence-dependent manner. The simplest possible distance map calculates
-Hamming distance between sequences without any site-specific weights, as shown
-below::
+Distance maps are defined in JSON format with two required top-level keys.
+The `default` key specifies the numeric (floating point) value to assign to all mismatches by default.
+The `map` key specifies a dictionary of weights to use for distance calculations.
+These weights are indexed hierarchically by gene name and one-based gene coordinate and are assigned in either a sequence-independent or sequence-dependent manner.
+The simplest possible distance map calculates Hamming distance between sequences without any site-specific weights, as shown below::
 
     {
         "name": "Hamming distance",
@@ -50,6 +47,25 @@ below::
         "map": {}
     }
 
+By default, distances are floating point values whose precision can be controlled with the `precision` key that defines the number of decimal places to retain for each distance.
+The following example shows how to specify a precision of two decimal places in the final output.::
+
+    {
+        "name": "Hamming distance",
+        "default": 1,
+        "map": {},
+        "precision": 2
+    }
+
+Distances can be reported as integer values by specifying an `output_type` as `integer` or `int` as follows.::
+
+    {
+        "name": "Hamming distance",
+        "default": 1,
+        "map": {},
+        "output_type": "integer"
+    }
+
 Sequence-independent distances are defined by gene and position using a numeric
 value of the same type as the default value (integer or float). The following
 example is a distance map for antigenic amino acid substitutions near influenza
@@ -135,6 +151,7 @@ import Bio
 import Bio.Phylo
 from collections import defaultdict
 import copy
+from itertools import chain
 import json
 import numpy as np
 import pandas as pd
@@ -198,7 +215,7 @@ def read_distance_map(map_file):
     return distance_map
 
 
-def get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map):
+def get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map, aggregate_function=max):
     """Calculate distance between the two given nodes using the given distance map.
 
     In cases where the distance map between sequences is asymmetric, the first
@@ -216,24 +233,37 @@ def get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
 
     Returns
     -------
-    int or float :
-        distance between node sequences based on the given map where the
-        returned type matches the type of the default value
+    float :
+        distance between node sequences based on the given map
 
     >>> node_a_sequences = {"gene": "ACTG"}
     >>> node_b_sequences = {"gene": "ACGG"}
     >>> distance_map = {"default": 0, "map": {}}
     >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
-    0
+    0.0
     >>> distance_map = {"default": 1, "map": {}}
     >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
-    1
+    1.0
     >>> distance_map = {"default": 0.0, "map": {"gene": {3: 1.0}}}
     >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
     0.0
-    >>> distance_map = {"default": 0.0, "map": {"gene": {2: 0.5, 3: 1.0}}}
+    >>> distance_map = {"default": 0.0, "map": {"gene": {2: 3.14159, 3: 1.0}}}
     >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
-    0.5
+    3.14159
+    >>> distance_map = {"default": 0.0, "precision": 2, "map": {"gene": {2: 3.14159, 3: 1.0}}}
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    3.14
+    >>> distance_map = {"default": 0.0, "output_type": "integer", "map": {"gene": {2: 3.14159, 3: 1.0}}}
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    3
+    >>> distance_map = {"default": 0.0, "output_type": "int", "map": {"gene": {2: 3.14159, 3: 1.0}}}
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    3
+    >>> distance_map = {"default": 0.0, "output_type": "unsupported", "map": {"gene": {2: 3.14159, 3: 1.0}}}
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    Traceback (most recent call last):
+        ...
+    ValueError: Unsupported output type of 'unsupported' provided in the distance map
 
     For site- and sequence-specific maps, the order of the input sequences
     matters; the first sequence is treated as the ancestral sequence while the
@@ -245,15 +275,138 @@ def get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
     >>> distance_map = {"default": 0.0, "map": {"gene": {2: {('T', 'G'): 0.5}}}}
     >>> get_distance_between_nodes(node_b_sequences, node_a_sequences, distance_map)
     0.0
+
+    Treat a single indel as one event.
+
+    >>> node_a_sequences = {"gene": "ACTG"}
+    >>> node_b_sequences = {"gene": "A--G"}
+    >>> distance_map = {"default": 1, "map": {}}
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    1.0
+
+    Use the maximum weight of all sites affected by an indel with a site-specific distance map.
+
+    >>> distance_map = {"default": 0, "map": {"gene": {1: 1, 2: 2}}}
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    2.0
+
+    Use the maximum weight of all mutations at all sites affected by an indel with a mutation-specific distance map.
+
+    >>> distance_map = {
+    ...     "default": 0,
+    ...     "map": {
+    ...         "gene": {
+    ...             1: {
+    ...                 ('C', 'G'): 1,
+    ...                 ('C', 'A'): 2
+    ...             },
+    ...             2: {
+    ...                 ('T', 'G'): 3,
+    ...                 ('T', 'A'): 2
+    ...             }
+    ...         }
+    ...     }
+    ... }
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    3.0
+
+    Use the maximum weight of gaps at all sites affected by an indel with a mutation-specific distance map.
+
+    >>> distance_map = {
+    ...     "default": 0,
+    ...     "map": {
+    ...         "gene": {
+    ...             1: {
+    ...                 ('C', '-'): 1,
+    ...                 ('C', 'A'): 2
+    ...             },
+    ...             2: {
+    ...                 ('T', 'G'): 3,
+    ...                 ('T', '-'): 2
+    ...             }
+    ...         }
+    ...     }
+    ... }
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    2.0
+
+    If the default value is greater than any of the site-specific mismatches and
+    the specific mismatch does not have a weighted defined, use the default
+    weight.
+
+    >>> distance_map = {
+    ...     "default": 4,
+    ...     "map": {
+    ...         "gene": {
+    ...             1: {
+    ...                 ('C', 'G'): 1,
+    ...                 ('C', 'A'): 2
+    ...             },
+    ...             2: {
+    ...                 ('T', 'G'): 3,
+    ...                 ('T', 'A'): 2
+    ...             }
+    ...         }
+    ...     }
+    ... }
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    4.0
+
+    Count mismatches adjacent to indel events.
+
+    >>> node_a_sequences = {"gene": "ACTGTA"}
+    >>> node_b_sequences = {"gene": "A--CCA"}
+    >>> distance_map = {"default": 1, "map": {}}
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    3.0
+
+    Ignore specific characters defined in the distance map.
+
+    >>> node_a_sequences = {"gene": "ACTGG"}
+    >>> node_b_sequences = {"gene": "A--GN"}
+    >>> distance_map = {"default": 1, "ignored_characters":["-"], "map": {}}
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    1.0
+    >>> distance_map = {"default": 1, "ignored_characters":["-", "N"], "map": {}}
+    >>> get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
+    0.0
+
     """
-    distance_type = type(distance_map["default"])
-    distance = distance_type(0)
+    distance = 0.0
+    ignored_characters = distance_map.get("ignored_characters", [])
+    output_type = distance_map.get("output_type", "float")
+    precision = distance_map.get("precision")
 
     for gene in node_a_sequences:
         gene_length = len(node_a_sequences[gene])
 
+        # In a first pass, find all mismatches between the given sequences. Use
+        # this pass to identify all sites in insertion/deletion (indel) events,
+        # so we can calculate an aggregate weight per event. Track each event by
+        # its start site.
+        mismatches_by_site = defaultdict(set)
+        in_gap = False
         for site in range(gene_length):
-            if node_a_sequences[gene][site] != node_b_sequences[gene][site]:
+            if (node_a_sequences[gene][site] != node_b_sequences[gene][site]
+                and node_a_sequences[gene][site] not in ignored_characters
+                and node_b_sequences[gene][site] not in ignored_characters):
+                if node_a_sequences[gene][site] == "-" or node_b_sequences[gene][site] == "-":
+                    if not in_gap:
+                        gap_start = site
+                        in_gap = True
+
+                    mismatches_by_site[gap_start].add(site)
+                else:
+                    in_gap = False
+                    mismatches_by_site[site].add(site)
+            else:
+                in_gap = False
+
+        # Sum distances across mismatched sites, aggregating indel events by a
+        # summary function (e.g., max, mean, etc.).
+        for sites in mismatches_by_site.values():
+            mismatch_distances = []
+            for site in sites:
                 if gene in distance_map["map"] and site in distance_map["map"][gene]:
                     # Distances can be provided as either site- and
                     # sequence-specific dictionaries of sequence pairs to
@@ -262,16 +415,46 @@ def get_distance_between_nodes(node_a_sequences, node_b_sequences, distance_map)
                     if isinstance(distance_map["map"][gene][site], dict):
                         seq_ancestral = node_a_sequences[gene][site]
                         seq_derived = node_b_sequences[gene][site]
-                        distance += distance_map["map"][gene][site].get(
-                            (seq_ancestral, seq_derived),
-                            distance_map["default"]
-                        )
-                    else:
-                        distance += distance_map["map"][gene][site]
-                else:
-                    distance += distance_map["default"]
 
-    return distance_type(np.round(distance, 2))
+                        # Check first for a user-defined weight for the
+                        # mismatched bases. This supports mismatch weights
+                        # between specific characters including gaps.
+                        if (seq_ancestral, seq_derived) in distance_map["map"][gene][site]:
+                            mismatch_distances.append(
+                                distance_map["map"][gene][site][(seq_ancestral, seq_derived)]
+                            )
+                        # Next, check whether the mismatch is a gap. We want to
+                        # take the aggregate of all weights at this site.
+                        elif seq_ancestral == "-" or seq_derived == "-":
+                            mismatch_distances.append(
+                                aggregate_function(
+                                    chain(
+                                        (distance_map["default"],),
+                                        distance_map["map"][gene][site].values()
+                                    )
+                                )
+                            )
+                        # Finally, use the default weight, if no
+                        # sequence-specific weights are defined.
+                        else:
+                            mismatch_distances.append(distance_map["default"])
+                    else:
+                        mismatch_distances.append(distance_map["map"][gene][site])
+                else:
+                    mismatch_distances.append(distance_map["default"])
+
+            # Aggregate the distances for all sites in the current mismatch.
+            distance += aggregate_function(mismatch_distances)
+
+    if output_type in ("integer", "int"):
+        return int(distance)
+    elif output_type != "float":
+        raise ValueError(f"Unsupported output type of '{output_type}' provided in the distance map")
+
+    if precision is not None:
+        distance = round(distance, precision)
+
+    return distance
 
 
 def get_distances_to_root(tree, sequences_by_node_and_gene, distance_map):
