@@ -6,6 +6,7 @@ import os, sys
 import time
 from collections import defaultdict, deque
 import warnings
+import numbers
 import re
 from Bio import Phylo
 from .utils import read_metadata, read_node_data, write_json, read_config, read_lat_longs, read_colors
@@ -211,8 +212,38 @@ def set_colorings(data_json, config, command_line_colorings, metadata_names, nod
         return key
 
     def _add_color_scale(coloring):
+        ## consider various sources to find any user-provided scale information
         key = coloring["key"]
-        if key.lower() in provided_colors:
+        scale_type = coloring["type"]
+        if scale_type == "continuous":
+            ## continuous scale information can only come from an auspice config JSON
+            if config.get(key, {}).get("scale"):
+                # enforce numeric values (we can't use the schema for this)
+                provided_scale = [s for s in config[key]['scale'] if isinstance(s[0], numbers.Number)]
+                if len(provided_scale)<2:
+                    warn(f"The scale provided for {key} had fewer than 2 (valid numeric) entries. Skipping.")
+                    return coloring
+                coloring["scale"] = provided_scale
+                return coloring
+        elif config.get(key, {}).get("scale"):
+            # If the auspice config JSON (`config`) explicitly defines a scale for this coloring
+            # then we use this instead of any colors provided via a TSV file (`provided_colors`)
+            values_in_tree = get_values_across_nodes(node_attrs, key)
+            scale = []
+            provided_values_unseen_in_tree = []
+            for info in config[key]['scale']:
+                if info[0] in values_in_tree:
+                    scale.append(info)
+                else:
+                    provided_values_unseen_in_tree.append(info[0])
+            if len(scale):
+                coloring["scale"] = scale
+                if len(provided_values_unseen_in_tree):
+                    warn(f"The configuration JSON specifies colors for \"{key}\" which aren't found in the tree:\n\t{', '.join(provided_values_unseen_in_tree)}.")
+                return coloring
+            warn(f"The configuration JSON specifies a color scale for {key} however none of the values in the tree are in this scale! Auspice will generate its own color scale for this trait.")
+        elif key.lower() in provided_colors:
+            # `provided_colors` typically originates from a colors.tsv file
             scale = []
             trait_values = {str(val).lower(): val for val in get_values_across_nodes(node_attrs, key)}
             trait_values_unseen = {k for k in trait_values}
@@ -223,9 +254,37 @@ def set_colorings(data_json, config, command_line_colorings, metadata_names, nod
             if len(scale):
                 coloring["scale"] = scale
                 if len(trait_values_unseen):
-                    warn("These values for trait {} were not specified in your provided color scale: {}. Auspice will create colors for them.".format(key, ", ".join(trait_values_unseen)))
-            else:
-                warn("You've specified a color scale for {} but none of the values found on the tree had associated colors. Auspice will generate its own color scale for this trait.".format(key))
+                    warn(f"These values for trait {key} were not specified in the colors file you provided:\n\t{', '.join(trait_values_unseen)}.\n\tAuspice will create colors for them.")
+                return coloring
+            warn(f"You've supplied a colors file with information for {key} but none of the values found on the tree had associated colors. Auspice will generate its own color scale for this trait.")
+        # Fallthrough (no scale information provided means that auspice will generate its own scale)
+        return coloring
+
+    def _add_legend(coloring):
+        """
+        If there is a config-defined legend entry for this coloring, then add it to
+        the coloring object under the key "legend"
+
+        Enclosing scope variables used
+        ------------------------------
+        config : object
+
+        Parameters
+        ----------
+        coloring : object
+
+        Returns
+        -------
+        object :
+            returns the single input parameter, potentially modified in place
+        """
+        key = coloring["key"]
+        if config.get(key, {}).get("legend"):
+            # The structure of the legend in the auspice config file is the same as the exported
+            # auspice dataset JSON, and the schema validates this for us so we don't have to check here.
+            # Note that if the data is inconsistent (e.g. overlapping bounds) then auspice will
+            # discard them (and print a console warning)
+            coloring['legend'] = config[key]['legend']
         return coloring
 
     def _add_title_and_type(coloring):
@@ -305,6 +364,8 @@ def set_colorings(data_json, config, command_line_colorings, metadata_names, nod
     colorings = [x for x in [_add_title_and_type(coloring) for coloring in colorings] if x]
     # for each coloring, if colors have been provided, save them as a "scale"
     colorings = [_add_color_scale(coloring) for coloring in colorings]
+    # for each coloring, pass-through the `legend` data from auspice_config.json, if provided
+    colorings = [_add_legend(coloring) for coloring in colorings]
     # save them to the data json to be exported
     data_json['meta']["colorings"] = colorings
 
@@ -433,6 +494,27 @@ def set_panels(data_json, config, cmd_line_panels):
     data_json['meta']["panels"] = panels
 
 
+def set_data_provenance(data_json, config):
+    """Set the data provenance from the given config file to the given data JSON.
+
+    Parameters
+    ----------
+    data_json : dict
+        auspice JSON to be updated
+    config : dict
+        config JSON with an expected ``data_provenance`` key
+
+    >>> config = {"data_provenance": [{"name": "GISAID"}, {"name": "INSDC"}]}
+    >>> data_json = {"meta": {}}
+    >>> set_data_provenance(data_json, config)
+    >>> data_json["meta"]["data_provenance"][0]["name"]
+    'GISAID'
+
+    """
+    if "data_provenance" in config:
+        data_json["meta"]["data_provenance"] = config["data_provenance"]
+
+
 def counter_to_disambiguation_suffix(count):
     """Given a numeric count of author papers, return a distinct alphabetical
     disambiguation suffix.
@@ -825,12 +907,7 @@ def set_description(data_json, cmd_line_description_file):
     except FileNotFoundError:
         fatal("Provided desciption file {} does not exist".format(cmd_line_description_file))
 
-def parse_node_data_and_metadata(T, node_data_files, metadata_file):
-    node_data = read_node_data(node_data_files) # node_data_files is an array of multiple files (or a single file)
-    if metadata_file is not None:
-        metadata, _ = read_metadata(metadata_file)
-    else:
-        metadata = {}
+def parse_node_data_and_metadata(T, node_data, metadata):
     node_data_names = set()
     metadata_names = set()
 
@@ -876,13 +953,25 @@ def run_v2(args):
     configure_warnings()
     data_json = {"version": "v2", "meta": {"updated": time.strftime('%Y-%m-%d')}}
 
+    #load input files
+    try:
+        node_data_file = read_node_data(args.node_data) # node_data_files is an array of multiple files (or a single file)
+    except FileNotFoundError:
+        print(f"ERROR: node data file ({args.node_data}) does not exist")
+        sys.exit(2)
+
+    if args.metadata is not None:
+        try:
+            metadata_file, _ = read_metadata(args.metadata)
+        except FileNotFoundError:
+            print(f"ERROR: meta data file ({args.metadata}) does not exist")
+            sys.exit(2)
+    else:
+        metadata_file = {}
+
     # parse input files
     T = Phylo.read(args.tree, 'newick')
-    try:
-        node_data, node_attrs, node_data_names, metadata_names = parse_node_data_and_metadata(T, args.node_data, args.metadata)
-    except FileNotFoundError:
-        print(f"ERROR: meta data file ({args.metadata}) does not exist")
-        sys.exit(2)
+    node_data, node_attrs, node_data_names, metadata_names = parse_node_data_and_metadata(T, node_data_file, metadata_file)
     config = get_config(args)
 
     # set metadata data structures
@@ -914,6 +1003,7 @@ def run_v2(args):
     set_node_attrs_on_tree(data_json, node_attrs)
     set_geo_resolutions(data_json, config, args.geo_resolutions, read_lat_longs(args.lat_longs), node_attrs)
     set_panels(data_json, config, args.panels)
+    set_data_provenance(data_json, config)
 
     # Write outputs - the (unified) dataset JSON intended for auspice & perhaps the ref root-sequence JSON
     indent = {"indent": None} if args.minify_json else {}
