@@ -82,8 +82,11 @@ def refine(tree=None, aln=None, ref=None, dates=None, branch_length_inference='a
 def collect_node_data(T, attributes):
     data = {}
     for n in T.find_clades():
-        data[n.name] = {attr:n.__getattribute__(attr)
-                        for attr in attributes if hasattr(n,attr)}
+        data[n.name] = {
+            attr: getattr(n, attr)
+            for attr in attributes
+            if getattr(n, attr, None) is not None
+        }
     return data
 
 
@@ -135,7 +138,7 @@ def run(args):
     # node data is the dict that will be exported as json
     node_data = {'alignment': args.alignment}
     # list of node attributes that are to be exported, will grow
-    attributes = ['branch_length']
+    attributes = ['branch_length', 'confidence']
 
     try:
         T = read_tree(args.tree)
@@ -222,7 +225,11 @@ def run(args):
                 node_data['skyline'] = [[float(x) for x in skyline.x], [float(y) for y in conf[0]],
                                         [float(y) for y in skyline.y], [float(y) for y in conf[1]]]
             except:
-                print("ERROR: skyline optimization by TreeTime has failed.", file=sys.stderr)
+                print(
+                    "ERROR: skyline optimization by TreeTime has failed.",
+                    "To avoid this error, try running without coalescent optimization or with `--coalescent opt` instead of `--coalescent skyline`.",
+                    file=sys.stderr
+                )
                 return 1
 
         attributes.extend(['numdate', 'clock_length', 'mutation_length', 'raw_date', 'date'])
@@ -238,7 +245,10 @@ def run(args):
             elif args.root in ['least-squares', 'min_dev', 'oldest']:
                 raise TypeError("The rooting option '%s' is only available when inferring a timetree. Please specify an explicit outgroup."%args.root)
             else:
-                T.root_with_outgroup(args.root)
+                try:
+                    T.root_with_outgroup(args.root)
+                except ValueError as err:
+                    raise ValueError(f"HINT: This error may be because your specified root with name '{args.root}' was not found in your alignment file") from err
 
         tt = TreeAnc(tree=T, aln=aln, ref=ref, gtr='JC69', verbose=1)
 
@@ -279,7 +289,7 @@ def run(args):
 
     # Export refined tree and node data
     import json
-    tree_success = Phylo.write(T, tree_fname, 'newick', format_branch_length='%1.8f')
+    tree_success = Phylo.write(T, tree_fname, 'newick', format_branch_length='%1.8f', branch_length_only=True)
     print("updated tree written to",tree_fname, file=sys.stdout)
 
     if args.output_node_data:
