@@ -285,7 +285,7 @@ def assign_aa_fasta(tree, translations):
 def get_genes_from_file(fname):
     genes = []
     if os.path.isfile(fname):
-        with open(fname) as ifile:
+        with open(fname, encoding='utf-8') as ifile:
             for line in ifile:
                 fields = line.strip().split('#')
                 if fields[0].strip():
@@ -308,7 +308,6 @@ def register_arguments(parser):
                         help='GenBank or GFF file containing the annotation')
     parser.add_argument('--genes', nargs='+', help="genes to translate (list or file containing list)")
     parser.add_argument('--output-node-data', type=str, help='name of JSON file to save aa-mutations to')
-    parser.add_argument('--output', '-o', type=str, help='DEPRECATED. Same as --output-node-data')
     parser.add_argument('--alignment-output', type=str, help="write out translated gene alignments. "
                                    "If a VCF-input, a .vcf or .vcf.gz will be output here (depending on file ending). If fasta-input, specify the file name "
                                    "like so: 'my_alignment_%%GENE.fasta', where '%%GENE' will be replaced by the name of the gene")
