@@ -1,10 +1,12 @@
 import datetime
+from pathlib import Path
 from unittest.mock import patch
 
 import pytest
 from freezegun import freeze_time
 
 from augur import utils
+from test_filter import write_metadata
 
 class TestUtils:
     def test_ambiguous_date_to_date_range_not_ambiguous(self):
@@ -129,3 +131,33 @@ class TestUtils:
         # Test incomplete date strings without ambiguous dates for the requested fields.
         assert not utils.is_date_ambiguous("2019", "year")
         assert not utils.is_date_ambiguous("2019-10", "month")
+
+    def test_read_strains(self, tmpdir):
+        # Write one list of filenames with some unnecessary whitespace.
+        strains1 = Path(tmpdir) / Path("strains1.txt")
+        with open(strains1, "w") as oh:
+            oh.write("strain1 # this is an inline comment about strain 1\nstrain2\n   # this is a comment preceded by whitespace.\n")
+
+        # Write another list of filenames with a comment.
+        strains2 = Path(tmpdir) / Path("strains2.txt")
+        with open(strains2, "w") as oh:
+            oh.write("# this is a comment. ignore this.\nstrain2\nstrain3\n")
+
+        strains = utils.read_strains(strains1, strains2)
+        assert len(strains) == 3
+        assert "strain1" in strains
+
+    def test_read_metadata(self, tmpdir):
+        meta_fn = write_metadata(tmpdir, (("strain","location","quality"),
+                                          ("SEQ_1","colorado","good"),
+                                          ("SEQ_2","colorado","bad"),
+                                          ("SEQ_3","nevada","good")))
+        utils.read_metadata(meta_fn, as_data_frame=True)
+        # duplicates SEQ_1 raises ValueError
+        meta_fn = write_metadata(tmpdir, (("strain","location","quality"),
+                                          ("SEQ_1","colorado","good"),
+                                          ("SEQ_1","colorado","bad"),
+                                          ("SEQ_3","nevada","good")))
+        with pytest.raises(ValueError) as e_info:
+            utils.read_metadata(meta_fn, as_data_frame=True)
+        assert str(e_info.value) == "Duplicated strain in metadata: SEQ_1"
